// $Id: ComObject.h,v 1.2 2000/04/22 21:39:36 chuang Exp $
#ifndef COMOBJECT_H
#define COMOBJECT_H

#pragma warning(disable: 4786)
#include <map>
#include "tcomApi.h"
#include "TclObject.h"
#include "TypeInfo.h"

// This class enables calls to COM object methods through a virtual
// function table.  The COM object methods are implemented by executing a
// Tcl command. 

class TCOM_API ComObject
{
    // Implement method invocation through virtual function table call.
    friend void __cdecl invokeComObjectFunction(
        volatile HRESULT hresult,
        volatile DWORD pArgEnd,
        DWORD ebp,
        DWORD funcIndex,
        DWORD retAddr,
        ComObject *pThis,
        ...);

    // pointer to virtual function table
    // This must be the first data member.
    const void *m_pVtbl;

    // count of references to this object
    long m_refCount;

    // interface description includes information about methods and properties
    const Interface &m_interface;

    // thread used to run Tcl interpreter
    Tcl_ThreadId m_threadId;

    // interpreter used to execute Tcl command
    Tcl_Interp *m_interp;

    // Tcl command executed to implement methods
    TclObject m_servant;

    // Tcl command executed when this COM object is destroyed
    TclObject m_destructor;

    // virtual function index to method description map
    typedef std::map<short, const Method *> VtblIndexToMethodMap;
    VtblIndexToMethodMap m_vtblIndexToMethodMap;

    // dispatch member ID to method description map
    typedef std::map<DISPID, const Method *> DispIdToMethodMap;
    DispIdToMethodMap m_dispIdToMethodMap;

    // true if this is actually an event sink
    bool m_isSink;

    // virtual function table for COM objects
    static const void *unknownVtbl[];

    // virtual function table for dispatchable objects
    static const void *dispatchVtbl[];

    // Do not allow others to create, copy or destroy instances of this class.
    ComObject(
        const Interface &interfaceDesc,
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject servant,
        TclObject destructor,
        bool isSink);

    ComObject(const ComObject &);       // not implemented
    void operator=(const ComObject &);  // not implemented

    ~ComObject();

public:
    // Create an instance of this class.
    static IUnknown *newInstance(
        const Interface &interfaceDesc,
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject servant,
        TclObject destructor,
        bool isSink=false);

    // IUnknown implementation
    static STDMETHODIMP QueryInterface(
        ComObject *pThis,
        REFIID riid,
        void **ppvObj);
    static STDMETHODIMP_(ULONG) AddRef(ComObject *pThis);
    static STDMETHODIMP_(ULONG) Release(ComObject *pThis);

    // IDispatch implementation
    static STDMETHODIMP GetTypeInfoCount(
        ComObject *pThis,
        UINT *pctinfo);
    static STDMETHODIMP GetTypeInfo(
        ComObject *pThis,
        UINT itinfo,
        LCID lcid,
        ITypeInfo **pptinfo);
    static STDMETHODIMP GetIDsOfNames(
        ComObject *pThis,
        REFIID riid,
        OLECHAR **rgszNames,
        UINT cNames,
        LCID lcid,
        DISPID *rgdispid);
    static STDMETHODIMP Invoke(
        ComObject *pThis,
        DISPID dispidMember,
        REFIID riid,
        LCID lcid,
        WORD wFlags,
        DISPPARAMS *pdispparams,
        VARIANT *pvarResult,
        EXCEPINFO *pexcepinfo,
        UINT *puArgErr);
};

#endif 
