// $Id: StringHashTable.h,v 1.1 2000/04/24 17:08:45 chuang Exp $
#ifndef STRINGHASHTABLE_H
#define STRINGHASHTABLE_H

#include <tcl.h>
#include "mutex.h"

// This is a hash table that maps strings to values of the given referent type.

template<class D>
class StringHashTable
{
    DECLARE_MUTEX(m_mutex)
    Tcl_HashTable m_hashTable;

public:
    typedef D referent_type;

    StringHashTable();

    ~StringHashTable();

    // Insert value into table.
    void insert(const char *key, D pValue);

    // Find value in table.
    D find(const char *key) const;

    // Remove value.
    void erase(const char *key);
};

template<class D>
StringHashTable<D>::StringHashTable ()
{
    Tcl_InitHashTable(&m_hashTable, TCL_STRING_KEYS);
}

template<class D>
StringHashTable<D>::~StringHashTable ()
{
    Tcl_DeleteHashTable(&m_hashTable);
}

// Insert a key and value.  If the key already exists in the table,
// replace the existing value.

template<class D>
void
StringHashTable<D>::insert (const char *key, D pValue)
{
    LOCK_MUTEX(m_mutex)

    int isNew;
    Tcl_HashEntry *pEntry = Tcl_CreateHashEntry(
        &m_hashTable,
        const_cast<char *>(key),
        &isNew);
    Tcl_SetHashValue(pEntry, reinterpret_cast<ClientData>(pValue));
}

template<class D>
D 
StringHashTable<D>::find (const char *key) const
{
    LOCK_MUTEX(m_mutex)

    Tcl_HashEntry *pEntry = Tcl_FindHashEntry(
        const_cast<Tcl_HashTable *>(&m_hashTable),
        const_cast<char *>(key));
    if (pEntry == 0) {
        return 0;
    }
    return reinterpret_cast<D>(Tcl_GetHashValue(pEntry));
}

template<class D>
void
StringHashTable<D>::erase (const char *key)
{
    LOCK_MUTEX(m_mutex)

    Tcl_HashEntry *pEntry = Tcl_FindHashEntry(
        &m_hashTable,
        const_cast<char *>(key));
    if (pEntry != 0) {
        Tcl_DeleteHashEntry(pEntry);
    }
}

#endif
