// $Id: TypeLib.h,v 1.12 2000/04/10 16:01:35 chuang Exp $
#ifndef TYPELIB_H
#define TYPELIB_H

#pragma warning(disable: 4786)
#include <vector>
#include <map>
#include <string>
#include "Uuid.h"
#include "TypeInfo.h"

// This describes a class.

class Class
{
    std::string m_name;
    CLSID m_clsid;
    const Interface *m_pDefaultInterface;
    const Interface *m_pSourceInterface;

public:
    Class(
        TypeInfo &typeInfo,
        const Interface *pDefaultInterface,
        const Interface *pSourceInterface);

    // Get name.
    std::string name () const
    { return m_name; }

    // Get CLSID.
    const CLSID &clsid () const
    { return m_clsid; }

    // Get CLSID as string.
    std::string clsidString () const
    { Uuid uuid(m_clsid); return uuid.toString(); }

    // Get default interface.
    const Interface *defaultInterface () const
    { return m_pDefaultInterface; }

    // Get source interface.
    const Interface *sourceInterface () const
    { return m_pSourceInterface; }
};

// This describes an enumeration with a map where
// the key is an enumerator name and
// the data is the enumerator value.

class Enum: public std::map<std::string, std::string>
{
    std::string m_name;

public:
    Enum(TypeInfo &typeInfo, TypeAttr &attr);

    // Get name.
    const std::string &name () const
    { return m_name; }
};

// This class represents a type library.

class TypeLib
{
public:
    typedef std::vector<const Interface *> Interfaces;
    typedef std::vector<Class> Classes;
    typedef std::vector<Enum> Enums;

private:
    ITypeLib *m_pITypeLib;
    Interfaces m_interfaces;
    Classes m_classes;
    Enums m_enums;

    TypeLib (ITypeLib *pITypeLib):
        m_pITypeLib(pITypeLib)
    { readTypeLib(); }

    // Do not allow others to copy instances of this class.
    TypeLib(const TypeLib &);           // not implemented
    void operator=(const TypeLib &);    // not implemented

    // Get information from type library.
    void readTypeLib();

public:
    ~TypeLib ()
    { m_pITypeLib->Release(); }

    // Load a type library and create a representation of it.
    static HRESULT load(
        const char *name, TypeLib *&pTypeLib, bool registerTypeLib=false);

    // Get type library name.
    std::string name() const;

    // Get type library documentation string.
    std::string documentation() const;

    // Get interfaces.
    const Interfaces &interfaces () const
    { return m_interfaces; }

    // Get the named interface.
    const Interface *findInterface(const char *name) const;

    // Get classes.
    const Classes &classes () const
    { return m_classes; }

    // Get the named class.
    const Class *findClass(const char *name) const;

    // Get enumerations.
    const Enums &enums () const
    { return m_enums; }

    // Get the named enumeration.
    const Enum *findEnum(const char *name) const;
};

#endif 
