// $Id: exemain.cpp,v 1.2 2000/04/26 15:58:45 chuang Exp $
#pragma warning(disable: 4786)
#include "ComModule.h"
#include "TclInterp.h"

// This class implements a COM module for an EXE server.

class ExeModule: public ComModule
{
    DWORD m_threadId;
    HANDLE m_shutdownEvent;

public:
    // Decrement lock count.
    virtual long unlock();

    // Wait for the shutdown event to be raised.
    void waitForShutdown();

    // Start thread waiting for shutdown event.
    bool startMonitor(DWORD threadId);
};

long
ExeModule::unlock()
{
    long count = ComModule::unlock();
    if (count == 0) {
        // Notify monitor to exit application.
        SetEvent(m_shutdownEvent);
    }
    return count;
}

void
ExeModule::waitForShutdown()
{
    while (true) {
        WaitForSingleObject(m_shutdownEvent, INFINITE);
        if (lockCount() == 0) {
            break;
        }
    }

    CloseHandle(m_shutdownEvent);
    PostThreadMessage(m_threadId, WM_QUIT, 0, 0);
}

// Passed to CreateThread to monitor the shutdown event.

static DWORD WINAPI
monitorProc (void *pv)
{
    ExeModule *pModule = reinterpret_cast<ExeModule *>(pv);
    pModule->waitForShutdown();
    return 0;
}

bool
ExeModule::startMonitor (DWORD threadId)
{
    m_threadId = threadId;

    m_shutdownEvent = CreateEvent(NULL, false, false, NULL);
    if (m_shutdownEvent == NULL) {
        return false;
    }

    DWORD myThreadId;
    HANDLE h = CreateThread(NULL, 0, monitorProc, this, 0, &myThreadId);
    return h != NULL;
}

extern "C" int WINAPI
WinMain (HINSTANCE /*hInstance*/,
         HINSTANCE /*hPrevInstance*/,
         LPTSTR /*lpCmdLine*/,
         int /*nShowCmd*/)
{
    ExeModule module;
    module.startMonitor(GetCurrentThreadId());

    // Evaluate script to register class factories.
    std::string filePath =
        TclInterp::instance().rootDir() + "/lib/tcom/factory.tcl";
    int completionCode = Tcl_EvalFile(
        TclInterp::instance(), const_cast<char *>(filePath.c_str()));
    if (completionCode != TCL_OK) {
        return completionCode;
    }

    MSG msg;
    while (GetMessage(&msg, 0, 0, 0)) {
        DispatchMessage(&msg);
    }
    
    module.revokeFactories();

    // Wait for any threads to finish.
    Sleep(1000);

    return 0;
}
