// $Id: main.cpp,v 1.39 2000/04/28 19:37:53 chuang Exp $
#include "ComInitializer.h"
#include "configureCmd.h"
#include "refCmd.h"
#include "bindCmd.h"
#include "releaseCmd.h"
#include "nullCmd.h"
#include "naCmd.h"
#include "infoCmd.h"
#include "importCmd.h"
#include "typelibCmd.h"
#include "objectCmd.h"

#define NAMESPACE "tcom::"
#define PACKAGE_NAME "tcom"
#define PACKAGE_VERSION "2.5"

#if 0
// Yes, this is an undocumented internal Tcl function, but this makes it
// easier for a native extension to export commands from a namespace.
EXTERN int Tcl_Export(Tcl_Interp *interp,
                      Tcl_Namespace *nsPtr,
                      char *pattern,
                      int resetListFirst);

// Export the command from its namespace.

static void
exportCommand (Tcl_Interp *interp, char *fullName, char *name)
{
    Tcl_CmdInfo cmdInfo;
    if (Tcl_GetCommandInfo(interp, fullName, &cmdInfo)) {
        Tcl_Export(interp, cmdInfo.namespacePtr, name, 0);
    }
}
#endif

/*
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 */
extern "C" DLLEXPORT int
Tcom_Init (Tcl_Interp *interp)
{
#if TCL_MINOR_VERSION >= 1
    // Stubs were introduced in Tcl 8.1.
    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    ComInitializer::concurrencyModel(COINIT_APARTMENTTHREADED);

    // Register new Tcl object types.
    new HandleSupport<Reference>(interp, "tcomReference", referenceObjCmd);
    new HandleSupport<Interface>(interp, "tcomInterface", interfaceObjCmd);
    new HandleSupport<TypeLib>(interp, "tcomTypeLib", typeLibObjCmd);
    Tcl_RegisterObjType(&naObjType);
    Tcl_RegisterObjType(&nullObjType);

    // Create additional commands.
    Tcl_CreateObjCommand(interp, NAMESPACE "bind", bindCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "class", classCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "configure", configureCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "import", importCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "info", infoCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "interface", interfaceCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "method", methodCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "na", naCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "null", nullCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "property", propertyCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "ref", refCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "release", releaseCmd, 0, 0);
    Tcl_CreateObjCommand(interp, NAMESPACE "typelib", typelibCmd, 0, 0);
#if TCL_MINOR_VERSION >= 1
    // Server objects require the thread support introduced in Tcl 8.1.
    Tcl_CreateObjCommand(interp, NAMESPACE "object", objectCmd, 0, 0);
#endif

    return Tcl_PkgProvide(interp, PACKAGE_NAME, PACKAGE_VERSION);
}

/*
 *	This procedure initializes commands for a safe interpreter.
 *	You would leave out of this procedure any commands you deemed unsafe.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	None.
 */
extern "C" DLLEXPORT int
Tcom_SafeInit (
    Tcl_Interp *interp)
{
#if TCL_MINOR_VERSION >= 1
    // Stubs were introduced in Tcl 8.1.
    if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    return Tcl_PkgProvide(interp, PACKAGE_NAME, PACKAGE_VERSION);
}
