// $Id: naCmd.cpp,v 1.2 1999/12/10 22:38:01 chuang Exp $
#include <string.h>
#include "naCmd.h"

// There is no internal representation to duplicate.

static void
naDupInternalRep (Tcl_Obj *pSrc, Tcl_Obj *pDup)
{ }

// The string representation is the same for all objects of this type.

static char naStringRep[] = "tcomNA";

static void
naUpdateString (Tcl_Obj *pObj)
{
    pObj->length = sizeof(naStringRep) - 1;
    pObj->bytes = Tcl_Alloc(pObj->length + 1);
    strcpy(pObj->bytes, naStringRep);
}

// Do not allow conversion from other types.

static int
naSetFromAny (Tcl_Interp *interp, Tcl_Obj *pObj)
{
    if (interp != NULL) {
        Tcl_AppendResult(interp, "cannot convert to ", naObjType.name, NULL);
    }
    return TCL_ERROR;
}

Tcl_ObjType naObjType = {
    naStringRep,
    NULL,
    naDupInternalRep,
    naUpdateString,
    naSetFromAny
};

// Create an NA object.

Tcl_Obj *
newNaObj ()
{
    Tcl_Obj *pObj = Tcl_NewObj();
    pObj->typePtr = &naObjType;
    return pObj;
}

// This Tcl command returns an object used to represent a missing optional
// argument.

int
naCmd (
    ClientData clientData,	/* Not used. */
    Tcl_Interp *interp,		/* Current interpreter. */
    int objc,			/* Number of arguments. */
    Tcl_Obj *CONST objv[])	/* The argument objects. */
{
    if (objc == 1) {
        // Return a missing argument token.
        Tcl_SetObjResult(interp, newNaObj());
        return TCL_OK;
    }

    static char *options[] = {
	"ismissing", NULL
    };
    enum SubCommandEnum {
        ISMISSING
    };

    int index;
    if (Tcl_GetIndexFromObj(interp, objv[1], options, "subcommand", 0,
     &index) != TCL_OK) {
        return TCL_ERROR;
    }

    switch (index) {
    case ISMISSING:
        // Return true if the object is a missing argument token.
        {
            if (objc != 3) {
	        Tcl_WrongNumArgs(interp, 2, objv, "object");
	        return TCL_ERROR;
            }

            Tcl_SetObjResult(
                interp,
                Tcl_NewBooleanObj(objv[2]->typePtr == &naObjType));
        }
	return TCL_OK;
    }
    return TCL_ERROR;
}
